/* asm/arch/memconfig.h */

#ifndef __ASM_ARCH_MEMCONFIG_H_
#define __ASM_ARCH_MEMCONFIG_H_

/*************************************************
 *                   FLASH
 *   1980:0000 +----------------+ (0000:0000)
 *             |     Loader     |      ( 32K)
 *   1980:8000 +----------------+ (0000:A000)
 *             | Reserve		|      ( 8K)
 *   1980:A000 +----------------+ (0000:C000)
 *             | HTTPS CA Data  |      ( 8K)
 *   1980:C000 +----------------+ (0000:C000)
 *             | MAC(lastest 6B)|      (  8K)
 *   1980:E000 +----------------+ (0000:E000)
 *             |  LOGO Image    |      (  8K)
 *   1981:0000 +----------------+ (0002:0000)
 *             |  Conf Data     |      ( 64K)
 *   1982:0000 +----------------+ (0002:0000)
 *             | Linux Kernel   |      (768K)
 *             |                |
 *   198E:0000 +----------------+ (000E:0000)
 *             | File System    |     (3200K)
 *             |                |
 *             |                |
 *   19BF:FFFF +----------------+ (003F:FFFF)
 *************************************************/

#define BSPCONF_FLASH_BASE 0x19800000
#define BSPCONF_FLASH_SIZE 0x00400000
#define FLASH_BASE BSPCONF_FLASH_BASE
#define FLASH_SIZE BSPCONF_FLASH_SIZE

#define BSPCONF_SDRAM_BASE 0x00000000 /* ?? */
#define BSPCONF_SDRAM_SIZE 0x02000000 /* 32MB */

/* FLASH Memory Map */
#define BSPCONF_FLASH_PARTITIONS        7	/* LDR+HTTPS_CA+MAC+CONF+LOGO+KNL+FS */

#define BSPCONF_BTLDR_FLASH_OFFSET      0x00000000
#define BSPCONF_BTLDR_FLASH_SIZE        0x00008000 /* 32KB Loader*/

#define BSPCONF_HTTPS_FLASH_OFFSET      0x0000A000
#define BSPCONF_HTTPS_FLASH_SIZE        0x00002000 /* 8KB HTTPS CA Data*/

#define BSPCONF_MAC_FLASH_OFFSET	0x0000C000
#define BSPCONF_MAC_FLASH_SIZE          0x00002000 /* 8KB MAC(lastest 6B)*/
#define BSPCONF_MAC6BYTE_OFFSET        (BSPCONF_MAC_FLASH_OFFSET+BSPCONF_MAC_FLASH_SIZE-6)

#define BSPCONF_LOGO_FLASH_OFFSET       0x0000E000
#define BSPCONF_LOGO_FLASH_SIZE         0x00002000 /* 8KB LOGO IMAGE Data*/

#define BSPCONF_CONF_FLASH_OFFSET     0x00010000
#define BSPCONF_CONF_FLASH_SIZE       0x000C0000 /* 64KB Config Data*/

#define BSPCONF_KERNEL_FLASH_OFFSET     0x00020000
#define BSPCONF_KERNEL_FLASH_SIZE       0x000C0000 /* 768KB */

#define BSPCONF_FS_FLASH_OFFSET         0x000E0000
#define BSPCONF_FS_FLASH_SIZE           0x00320000 /* 3200KB */


#endif /* __ASM_ARCH_MEMCONFIG_H_ */
