
/***************************************************************
 * (C) COPYRIGHT SerComm Corporation All Rights Reserved, 2004.
 *
 *      Use of this software is restricted to the terms and
 *      conditions of SerComm's software license agreement.
 *
 *                        www.sercomm.com
 ****************************************************************/
#ifndef _FL_CONF_H_
#define _FL_CONF_H_

#include "sc_config.h"

//error code
#define	FL_RangeErr	1
#define FL_IOErr	2
#define FL_MemErr	3
#define FL_NullPara	4
#define FL_EraseErr	5
#define FL_FileErr	6
#define FL_InvalidMac	7

#define FLASH_SUCCESS	0
#define FLASH_FAILURE   FL_IOErr	//ioctl error

//-----------------------------------------------------------
// purpose: read configuration from flash to ramdisk file
// input: fname - ramdisk file name
// return: 0 - ok 
//         other values - read error
//-----------------------------------------------------------
int fl_read_conf(char *fname);

//-----------------------------------------------------------
// purpose: write configuration from file to flash
// input: fname - configuration file name
// return: 0 - ok 
//         other values - write error
//-----------------------------------------------------------
int fl_write_conf(char *fname);

//	Location 0x20
/*
Domain
	USA: 0x10
	Canada:0x20
	Most of Europe:0x30
	Spin:0x31
	France:0x32
	UK:0x33
	German:0x34
	Austrialia:0x35
	Japan:0x40
	Japan All:0x41
	S Chinese:0x70
	T Chisese:0x80
	Korea:0x90

Language
	Simplified Chinese (CN):0x00
	English (EN):0x01
	French (FR):0x02
	German (DE):0x03
	Swedish_SE (SV):0x04
	Spanish (ES):0x05
	Italian (IT):0x06
	Portugal (PT):0x07
	Denmark (DK):0x08
Dutch (DU):0x09
	America English (US):0x14
	Simplified Chinese (SC):0x15
	Traditional Chinese (TC):0x16
	Australia English (AU):0x17
	Japanese (JP):0x18
	Korean (KR):0x19
 */
typedef struct
{
	//char header[32];
	char domain;
	char language[4];
	char reserve[3];
	char serial[32];
	char pin[8];
	char rc4_key[64];
	char pid[24];
	char vid[8];
	char entity_name[256];
	char product_desp[256];
	char time_creation[12];	// YYYYMMDDHHMM
	char boot_mode;		// 0: non-specified, 1: force standalone mode, 2: force CSIBU mode
} MacInfo __attribute__((packed));
int fl_read_mac_info(MacInfo *mac_info);

#ifdef _WPA_WPA2_ENTERPRISE_
//-----------------------------------------------------------
// purpose: read CA from flash to ramdisk file
// input: fname - ramdisk file name
//	  type	- 0: TLS root CA,1: TLS user CA+PrivateKey,2:TTLS root CA
// return: 0 - ok 
//         other values - read error
//-----------------------------------------------------------
int fl_read_CA(char *fname,int type);

//-----------------------------------------------------------
// purpose: write CA from file to flash
// input: fname - configuration file name
//	  type	- 0: TLS root CA,1: TLS user CA+PrivateKey,2:TTLS root CA
// return: 0 - ok 
//         other values - write error
//-----------------------------------------------------------
int fl_write_CA(char *fname,int type);
#endif

//--------------------------------------------------------------
// purpose: read mac address from flash
// output: mac - buffer to save mac address (6 bytes)
//               if the mac address==00c002012345 then mac[0]== 
//		 0x00c0, mac[1]==0x0201 and mac[2]==0x2345
// return: 0 - ok 
//         other values - write error
//--------------------------------------------------------------
//int fl_read_mac(unsigned short mac[3]);

//--------------------------------------------------------------
// purpose: write mac address to flash
// input: mac - mac address (6 bytes)
//              if the mac address==00c002012345 then mac[0]== 
//		0x00c0, mac[1]==0x0201 and mac[2]==0x2345
// return: 0 - ok 
//         other values - write error
//--------------------------------------------------------------
//int fl_write_mac(unsigned short mac[3]);

//---------------------------------------------------------
//purpuse: check cfg magic number match to def_sys.conf
//input: fname - file to checked
//return: 0 - magic number match
//        other value - magic number not match
//----------------------------------------------------------
int check_magic_number(char *fname);


#endif
