
/***************************************************************
 * (C) COPYRIGHT SerComm Corporation All Rights Reserved, 2005.
 *
 *      Use of this software is restricted to the terms and
 *      conditions of SerComm's software license agreement.
 *
 *                        www.sercomm.com
 ****************************************************************/

#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "flash.h"
#include "PRO_file.h"
#include "conf_sec.h"

#define	dprintf	printf

/*************************************************
 *                   FLASH			 
 *   0010:0000 +----------------+ (0000:0000)  
 *             |     Loader     |      ( 56K) 	
 *             | MAC(lastest 6B)|      	
 *   0010:E000 +----------------+ (0000:E000)  
 *             |  Config Data   |      (  8K)
 *   0011:0000 +----------------+ (0001:0000) 	
 *             | Linux Kernel   |      (896K)	
 *             |                |
 *   001F:0000 +----------------+ (000F:0000)
 *             | File System    |     (3144K)
 *             |                |
 *             |                |
 *   004F:FFFF +----------------+ (003F:FFFF) (003F:FFFF)
 *************************************************/

//*********************************************************************
// Flow to control flash
// 1. InitFlashData	(optional, will be called automatically)
// 2. flash_read(), or flash_write() or flash_erase() or 
//    flash_erase_block()
// 3. flash_close()
//*********************************************************************
#define FL_BUF_SIZE     1024

typedef struct{
   unsigned char *startaddr;
   unsigned char *endaddr;
   unsigned int  len;   
}fl_data;

fl_data FlashData[TOTAL_SECT];
static int FlashInit=0,FlashFd=-1;

#define SIGNATURE		0x55aa

//######### The size of conf_blk must be 16 bytes !!! ###########
typedef struct{
unsigned short signature;	//signature =0x55aa
unsigned short dump1[3];	//6 bytes
unsigned long len;		//configure file length (4 bytes)
unsigned short dump2[2];	//4 bytes

}conf_blk;
//#if sizeof(conf_blk) neq 16 
//static unsigned short DefMac[3]={0x00C0,0x0200,0x0000};
//============================================================

int InitFlashData(void)
{
    //int i,j,blockSize;

    //dprintf("InitFlashData\n");
    if(FlashFd==-1)
       FlashFd = open(MTD_ALLFLASH,O_RDWR);

    FlashInit=1;
    return(FLASH_SUCCESS); 
}

//----------------------------------------------------------------
// Note: user must erase flash before writing !!!
//----------------------------------------------------------------
int flash_write(unsigned char *addr,unsigned char *buff,unsigned int Len, unsigned int *actLen)
{
   int i=0;

   if(Len==0) return(FLASH_SUCCESS);
   if((unsigned int)(addr+Len)>(unsigned int)FLASH_SIZE) return(FL_RangeErr);
   if(!FlashInit || FlashFd==-1){
      i=InitFlashData();
      if(i!=FLASH_SUCCESS) return(i);
   }
   lseek(FlashFd,(off_t)addr,SEEK_SET);
   //printf("flash addr  %08x\n",addr);
   *actLen = write(FlashFd,buff, Len);
   if(*actLen>=Len)
      return(FLASH_SUCCESS); 
   else{
      //dprintf("flash_write %lx error len=%d actLen=%d\n",(unsigned long)addr,Len,*actLen);
      return(FLASH_FAILURE);
   }
}
int flash_read(unsigned char * addr,  // Of flash.
                unsigned char *buff, // Destination buffer.
                unsigned int Len, unsigned int *actLen)
{
   int i=0;
   off_t offset;

   //printf("\n** flashReadBlock %lx len=%d\n",(unsigned long)addr,Len);
   if(Len==0) return(FLASH_SUCCESS);
   if((unsigned int)(addr+Len)>(unsigned int)FLASH_SIZE)
      return(FL_RangeErr);
   if(!FlashInit || FlashFd==-1){
      i=InitFlashData();
      if(i!=FLASH_SUCCESS) return(i);
   }
   //printf("FlashFd=%d\n",FlashFd);
   offset=lseek(FlashFd,(off_t)addr,SEEK_SET);
   if(offset!=(off_t)addr){
	//dprintf("lseek error %s\n",strerror(errno));
	return(FLASH_FAILURE);
   }	
   *actLen = read(FlashFd,buff, Len);
   if(*actLen>=Len){
      //printf("##flashReadBlock OK\n");	
      return(FLASH_SUCCESS); 
   }
   else{
	//dprintf("read %lx len=%d actLen=%d\n",(unsigned long)addr,Len,*actLen);
      return(FLASH_FAILURE);
   }
}
void flash_close(void)
{
   if(FlashFd!=-1){
	close(FlashFd);
	FlashFd=-1;
   }
   FlashInit=0;

}

int chk_len(unsigned long len)
{
   unsigned long num;

   num=sizeof(conf_blk);
   if(num & 1)
      num++;
   if (len>FlashData[CFG_SECT].len-num)
      return(1);
   else
      return(0);
}
int fl_write_conf(char *fname)
{
   int ConfFd;
   struct stat  st;
   unsigned int num,retNum;
   unsigned char *offset;
   conf_blk blk;
   char *p;
   FILE *fh;
   //int ccode;


   //dprintf("Write configuration to flash...\n");
   if(stat(fname,&st)){
     //dprintf("Fail to get file length of %s\n",fname);
     return(FL_FileErr);
   }
   //dprintf("st_size=%ld\n",st.st_size);
   if(chk_len(st.st_size))
      return(FL_RangeErr);

   p=malloc(sizeof(char)*FL_BUF_SIZE);
   if(!p){
      return(FL_MemErr);
   }
   blk.len=st.st_size;
   //dprintf("File size of %s =%ld 0x%lx\n",fname,blk.len,blk.len);
   fh=fopen(fname,"r");
   if(!fh){
      free(p);
      return(FL_FileErr);
   }
   
    ConfFd = open(MTD_FLASH_CONF,O_RDWR);

   //write block header
   blk.signature=0xffff;
   num=sizeof(blk);
   offset=0;
   //dprintf("Write config block header to flash offset=%04lX %d bytes\n",(long)offset,num);
   retNum = write(ConfFd, (unsigned char *)&blk, num);
   //dprintf("ccode=%d retNum=%d\n",ccode,retNum);
   if(retNum<num){
      free(p);	
      fclose(fh);
      close(ConfFd);
      return -1;
   }
   offset+=num;
   if((unsigned long)offset & 1)
      offset++;

   do{
     num=fread(p,sizeof(char),FL_BUF_SIZE,fh);
     if(num==0)
        break;
     //dprintf("Write file data to flash offset=%lx %d bytes\n",(long)offset,num);
    retNum = write(ConfFd, (unsigned char *)p, num);
     //dprintf("ccode=%d retNum=%d\n",ccode,retNum);
     if(retNum<num){
        free(p);	
        fclose(fh);
	close(ConfFd);
        return(-1);
     }
     offset+=num;

   }while(num==FL_BUF_SIZE);

   fclose(fh);
   free(p);

   //write signature
   blk.signature=SIGNATURE;
   num=sizeof(blk.signature);
   lseek(ConfFd,0,SEEK_SET);
   //dprintf("Write signature to flash offset=%lx %d bytes\n",(unsigned long)offset,num);
   retNum = write(ConfFd, (unsigned char *)&blk.signature, num);
   close(ConfFd);
   return 0;
}
int get_default_conf(char *fname)
{
   FILE *fh,*fh0;
   char tmp[128];
   size_t i;

   fh0=fopen(DEF_CONF_FILE,"r");
   if(!fh0){
      return(FL_FileErr);
   }
   fh=fopen(fname,"w");
   if(!fh){
      fclose(fh0);
      //dprintf("Can't open file %s\n",fname);
      return(FL_FileErr);
   }
   do{
     i=fread(tmp,sizeof(char),128,fh0);
     fwrite(tmp,sizeof(char),i,fh);
   }while(i==128);
   fclose(fh0);
   fclose(fh);
   return(0);
}
//---------------------------------------------------------
//purpuse: check cfg magic number match to def_sys.conf
//input: fname - file to checked
//return: 0 - magic number match
//        other value - magic number not match
//----------------------------------------------------------
int check_magic_number(char *fname)
{
   FILE *fh;
   char key1[15],key2[15];

   memset(key1,0,15);
   memset(key2,0,15);
   fh=fopen(DEF_CONF_FILE,"r");
   if(!fh){
      return(0);	//no default file, just return 0
   }

   PRO_GetStr(SEC_SYS,SYS_CFG_VER,key1,15,fh);
   fclose(fh);
   fh=fopen(fname,"r");
   if(!fh){
      return(-1);
   }

   PRO_GetStr(SEC_SYS,SYS_CFG_VER,key2,15,fh);
   fclose(fh);
   //dprintf("check_magic_number def_sys.conf=%s %s=%s\n",key1,fname,key2);
   return(strcmp(key1,key2));
}
//==================================================================
// NOTE: Because fl_write_conf will call flash_close(). I only call
//       flash_close() if necessary
//==================================================================
int fl_read_conf(char *fname)
{
   int ConfFd; 
   FILE *fh;
   unsigned int num,retNum=0;
   unsigned char *offset;
   conf_blk blk;
   char *p,buf[80];

    ConfFd = open(MTD_FLASH_CONF,O_RDWR);
   //dprintf("Read configuration from flash...\n");
   offset=0;
   num=sizeof(blk); 
   //dprintf("Read block header from offset=%04lX len=%d\n",(long)offset,num);
   retNum=read(ConfFd ,(unsigned char *)&blk,num);
   //dprintf("signature=%x len=%ld\n",blk.signature,blk.len);
   if(blk.signature!=SIGNATURE || chk_len(blk.len)){
   		close(ConfFd);
        get_default_conf(fname); 
        return(0);
         	
   }
   //dprintf("Signature=%x file len=%ld\n",blk.signature,blk.len);
   offset+=num;
   if((unsigned long )offset & 1)
      offset++;
   p=malloc(sizeof(char)*FL_BUF_SIZE);
   if(!p){
      //dprintf("Memory allocate failure\n");
      close(ConfFd);
      return(FL_MemErr);
   }
   fh=fopen(fname,"w");
   if(!fh){
      free(p);
      //dprintf("Can't open file %s\n",fname);
	close(ConfFd);
      return(FL_FileErr);
   }
   while(blk.len>0){
      num=(blk.len>FL_BUF_SIZE)?FL_BUF_SIZE:blk.len;
      retNum=read(ConfFd, (unsigned char *)p, num);
      fwrite(p,sizeof(char),num,fh);
      blk.len-=num;
      offset+=num;

   }
   fclose(fh);
   free(p);
   if (check_magic_number(fname)==0){
      close(ConfFd);
      return(0);
   }
   close(ConfFd);
   //dprintf("magic number not match, copy def_sys.conf to %s\n",fname);
   //magic number not match reset to factory defaults
   sprintf(buf,"cp %s %s >/dev/null 2>/dev/null",DEF_CONF_FILE,fname);
   system(buf);
   return(0);

}

int fl_read_mac_info(MacInfo *mac_info)
{
	int ConfFd; 
	int ret;
	int num = sizeof(MacInfo);

	if (-1 == (ConfFd = open(MTD_MAC, O_RDONLY)))
	{
		return -1;
	}
	ret = read(ConfFd, mac_info, num);
	close(ConfFd);
	if (ret != num)
	{
		return -2;
	}
	return(0);
}

#ifdef _WPA_WPA2_ENTERPRISE_
#define CA_EACH_SIZE		0x1000		// 4K (8K/2)
int fl_read_CA(char *fname,int type)
{
	int ConfFd; 
	FILE *fh;
   unsigned int num,retNum=0;
	unsigned char *offset;
	conf_blk blk;
	int seek_len = type*CA_EACH_SIZE;
	//char *p,buf[80];
	char *p;

	ConfFd = open(MTD_RESERVE,O_RDWR);
	if (ConfFd < 0)
	{
		return -1;
	}
	lseek(ConfFd, seek_len, SEEK_SET);
	
	offset=0;
	num=sizeof(blk); 
	//dprintf("Read block header from offset=%04lX len=%d\n",(long)offset,num);
	retNum=read(ConfFd ,(unsigned char *)&blk, num);
	if (retNum != num)
	{
		close(ConfFd);
		return -1;
	}
	offset+=num;
	p=malloc(sizeof(char)*FL_BUF_SIZE);
	if(!p)
	{
		//dprintf("Memory allocate failure\n");
		close(ConfFd);
		return(FL_MemErr);
	}

	if(blk.signature!=SIGNATURE) 
	{
		free(p);
		close(ConfFd);
		return -1;
	}
	if((unsigned long )offset & 1)
	{
		offset++;
	}
	fh=fopen(fname, "w");
	if(!fh)
	{
		free(p);
		close(ConfFd);
		return -1;
	}
	while(blk.len>0)
	{
		num=(blk.len>FL_BUF_SIZE)?FL_BUF_SIZE:blk.len;
		read(ConfFd, (unsigned char *)p, num);
		fwrite(p,sizeof(char),num,fh);
		blk.len-=num;
		offset+=num;
	}
	fclose(fh);
	free(p);
	close(ConfFd);
	return 0;
}

int fl_write_CA(char *fname,int type)
{
	int ConfFd;
	struct stat  st;
	unsigned int num,retNum;
	unsigned char *offset;
	conf_blk blk;
	char *p;
	FILE *fh;
	int seek_len = type*CA_EACH_SIZE;
	
	if(stat(fname,&st))
	{
		return(FL_FileErr);
	}

	blk.len=st.st_size;
	if (blk.len > (CA_EACH_SIZE-(sizeof(blk)+4)))
	{
		return -1;
	}
	p=malloc(sizeof(char)*FL_BUF_SIZE);
	if(!p)
	{
		return(FL_MemErr);
	}
	fh=fopen(fname,"r");
	if(!fh)
	{
		free(p);
		return(FL_FileErr);
	}

	ConfFd = open(MTD_RESERVE,O_RDWR);
	lseek(ConfFd, seek_len, SEEK_SET);

	//write block header
	blk.signature=0xffff;
	num=sizeof(blk);
	offset=0;
	//dprintf("Write config block header to flash offset=%04lX %d bytes\n",(long)offset,num);
	retNum = write(ConfFd, (unsigned char *)&blk, num);
	//dprintf("ccode=%d retNum=%d\n",ccode,retNum);
	if(retNum<num)
	{
		free(p);	
		fclose(fh);
		close(ConfFd);
		return -1;
	}
	offset+=num;
	if((unsigned long)offset & 1)
	{
		offset++;
	}
	do
	{
		num=fread(p,sizeof(char),FL_BUF_SIZE,fh);
		if(num==0)
		{
			break;
		}
		//dprintf("Write file data to flash offset=%lx %d bytes\n",(long)offset,num);
		retNum = write(ConfFd, (unsigned char *)p, num);
		//dprintf("ccode=%d retNum=%d\n",ccode,retNum);
		if(retNum<num)
		{
			free(p);	
			fclose(fh);
			close(ConfFd);
			return(-1);
		}
		offset+=num;

	}while(num==FL_BUF_SIZE);

	fclose(fh);
	free(p);

	//write signature
	blk.signature=SIGNATURE;
	num=sizeof(blk.signature);
	lseek(ConfFd, seek_len, SEEK_SET);
	//dprintf("Write signature to flash offset=%lx %d bytes\n",(unsigned long)offset,num);
	retNum = write(ConfFd, (unsigned char *)&blk.signature, num);
	close(ConfFd);
	return 0;
}
#endif

