/***************************************************************
 * (C) COPYRIGHT SerComm Corporation All Rights Reserved, 2004.
 *
 *      Use of this software is restricted to the terms and
 *      conditions of SerComm's software license agreement.
 *
 *                        www.sercomm.com
 ****************************************************************/
#include <stdio.h>		/* printf, fprintf, fopen, fclose */
#include <sys/types.h>		/* open, lssek */
#include <sys/stat.h>		/* open */
#include <fcntl.h>		/* open */
#include <unistd.h>		/* lseek, read */
#include <stdlib.h>		/* system, malloc */
#include <string.h>		/* memcmp, strcmp */
#include <time.h>		/* time, localtime */
#include "SCHD_functions.h"

#define SCHD_EXE_SCHED_START "/usr/sbin/crond &     >/dev/null 2>/dev/null"
#define SCHD_EXE_SCHED_STOP  "/usr/bin/killall crond>/dev/null 2>/dev/null"
#if 0
#define SCHD_CONF_FILENAME   "/mnt/ramfs/crontab"
#else
#define SCHD_CONF_FILENAME   "/mnt/ramdisk/crontab"
#endif
#define SCHD_CONF_TMPFILENAME "/tmp/crontab."

/* 00-99/99 * * * * root Put_Your_Command_Line_Here! */
/* <00-99/99>*5 + <SPACE>*6 + root + CMD[60] + END<\00> */
#define SCHD_CONF_TASK_OWNER "root"
#define SCHD_CONF_LINE_LEN   ((8*5) + 6 + strlen(SCHD_CONF_TASK_OWNER) + SCHD_MAX_CMDLINE_LEN + 1)
#define SCHD_CONF_ANY_STRING "*"

#define SCHD_MAX_BUFFER 256	/* should enough?? */
static char rw_buffer[SCHD_MAX_BUFFER]; /* global buffer */

/*
 * TOOL Functions
 */
/* ---- The input data MUST between MIN and MAX (equal to) ---- */
int SCHDCheckRange(int value, int min, int max)
{
	if (value < min) {
		return SCHD_ERROR;
	} /* too "small" */
	if (value > max) {
		return SCHD_ERROR;
	} /* too "large" */
	return SCHD_OK;
}

/* ---- Do the simple error check to the input data ---- */
int SCHDCheckField(SchdInfo_t *info)
{
	if (strlen(info->command) == 0) {
		return SCHD_ERROR; /* No Command? How does it work? */
	} else if (strlen(info->command) > SCHD_MAX_CMDLINE_LEN) {
		return SCHD_ERROR; /* Command-Line too LONG */
	} else {
		info->command[SCHD_MAX_CMDLINE_LEN-1] = '\0';
	} /* force to pad \00 in the End-Of-String */
				/* week day */
	if (info->week.interval != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->week.interval, 0, 6) != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->week.start != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->week.start, 0, 6)    != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->week.stop != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->week.stop, 0, 6)     != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
				/* month */
	if (info->month.interval != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->month.interval, 1, 12) != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->month.start != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->month.start, 1, 12)    != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->month.stop != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->month.stop, 1, 12)     != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
				/* month day */
	if (info->day.interval != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->day.interval, 1, 31) != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->day.start != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->day.start, 1, 31)    != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->day.stop != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->day.stop, 1, 31)     != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
				/* hour */
	if (info->hour.interval != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->hour.interval, 0, 23) != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->hour.start != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->hour.start, 0, 23)    != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->hour.stop != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->hour.stop, 0, 23)     != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
				/* minute */
	if (info->min.interval != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->min.interval, 0, 59) != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->min.start != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->min.start, 0, 59)    != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	if (info->min.stop != SCHD_DONTCARE) {
		if (SCHDCheckRange(info->min.stop, 0, 59)     != SCHD_OK) {
			return SCHD_ERROR;
		}
	}
	return SCHD_OK;
}

/* ---- Convert the user input data into the crontab format ---- */
/* ---- The input buffer length MUST .ge. SCHD_CONF_LINE_LEN ---- */
int SCHDConvtUserFormat(SchdInfo_t *info, char *buffer, int buf_len)
{
	if (buf_len < SCHD_CONF_LINE_LEN) {
		return SCHD_EMEM;
	}
	memset(buffer, '\0', buf_len); /* init */
				/* min */
	if (info->min.start == SCHD_DONTCARE) {
		sprintf(buffer, "%s", SCHD_CONF_ANY_STRING);
	} else {
		sprintf(buffer, "%d", info->min.start);
		if (info->min.stop != SCHD_DONTCARE) {
			sprintf(buffer, "%s-%d", buffer, info->min.stop);
			if (info->min.interval != SCHD_DONTCARE) {
				sprintf(buffer, "%s/%d", buffer, info->min.interval);
			} /* 00-99/99 */
		} /* 00-99 */
	} /* 00 */
				/* hour */
	if (info->hour.start == SCHD_DONTCARE) {
		sprintf(buffer, "%s %s", buffer, SCHD_CONF_ANY_STRING);
	} else {
		sprintf(buffer, "%s %d", buffer, info->hour.start);
		if (info->hour.stop != SCHD_DONTCARE) {
			sprintf(buffer, "%s-%d", buffer, info->hour.stop);
			if (info->hour.interval != SCHD_DONTCARE) {
				sprintf(buffer, "%s/%d", buffer, info->hour.interval);
			} /* 00-99/99 */
		} /* 00-99 */
	} /* 00 */
				/* day */
	if (info->day.start == SCHD_DONTCARE) {
		sprintf(buffer, "%s %s", buffer, SCHD_CONF_ANY_STRING);
	} else {
		sprintf(buffer, "%s %d", buffer, info->day.start);
		if (info->day.stop != SCHD_DONTCARE) {
			sprintf(buffer, "%s-%d", buffer, info->day.stop);
			if (info->day.interval != SCHD_DONTCARE) {
				sprintf(buffer, "%s/%d", buffer, info->day.interval);
			} /* 00-99/99 */
		} /* 00-99 */
	} /* 00 */
				/* month */
	if (info->month.start == SCHD_DONTCARE) {
		sprintf(buffer, "%s %s", buffer, SCHD_CONF_ANY_STRING);
	} else {
		sprintf(buffer, "%s %d", buffer, info->month.start);
		if (info->month.stop != SCHD_DONTCARE) {
			sprintf(buffer, "%s-%d", buffer, info->month.stop);
			if (info->month.interval != SCHD_DONTCARE) {
				sprintf(buffer, "%s/%d", buffer, info->month.interval);
			} /* 00-99/99 */
		} /* 00-99 */
	} /* 00 */
				/* week */
	if (info->week.start == SCHD_DONTCARE) {
		sprintf(buffer, "%s %s", buffer, SCHD_CONF_ANY_STRING);
	} else {
		sprintf(buffer, "%s %d", buffer, info->week.start);
		if (info->week.stop != SCHD_DONTCARE) {
			sprintf(buffer, "%s-%d", buffer, info->week.stop);
			if (info->week.interval != SCHD_DONTCARE) {
				sprintf(buffer, "%s/%d", buffer, info->week.interval);
			} /* 00-99/99 */
		} /* 00-99 */
	} /* 00 */
				/* owner (root) */
	sprintf(buffer, "%s %s", buffer, SCHD_CONF_TASK_OWNER);
				/* task command-line */
	sprintf(buffer, "%s %s\n", buffer, info->command);
	return SCHD_OK;
}

/* ---- Convert the crontab format into the structure format ---- */
// int SCHDConvtConfFormat(SchdInfo_t *info, void *buf, int buf_len)
// {		<< * -- We don't support it, yet! -- * >>
// }

/* ---- Like the shell "cut" command. It will cut the input buffer with ---- */
/* ---- <SPACE> delimiter and try to get the Xth field (number). If not ---- */
/* ---- enough delimiters (not found): return SCHD_ERRO, others return  ---- */
/* ---- the buffer pointer back.                                        ---- */
int SCHDCutFields(char *in_buff, int length, int number, char *out_buff)
{
	int i, count=0;

	if ((length <= 0)||(number < 0)) {
		return SCHD_ERROR;
	}
	if (number == 0) {
		return SCHD_OK;
	} /* the first field: no change. The same buffer position! */
	for (i=0; i<length; i++) {
		if (in_buff[i] == ' ') {
			count++;
		}
		if (count == number) {
			break;
		}
	}
	if (i < (length-1)) {	/* != length and (length-1) */
				/* in_buff[i]=' ', return NEXT. */
		memcpy(out_buff, &in_buff[i+1], (length-i)-1);
 		if (out_buff[strlen(out_buff)-1] == '\n') {
			out_buff[strlen(out_buff)-1] = '\0';
		} /* need to truncate the last "\n" */
		return SCHD_OK;
	}
	return SCHD_ERROR;
}

/* ---- Use current timestamp to generate a random number (4 digits) ---- */
/* ---- The upper layer MUST prepare at least 4+1 bytes for buffer!  ---- */
int SCHDGetRandom4(char *buffer)
{
	time_t curtime;
	struct tm *loctime;

	memset(buffer, '\0', 5); /* MMSS\00 */
	curtime = time(NULL);
	loctime = localtime(&curtime);
	/* -- buffer length = MM/DD/YYYY HH:MM:SS = 20 (+\00) -- */
	strftime(buffer, 4, "%M%S", loctime); /* Minute:Second */
	return SCHD_OK;
}

/*
 * Exported Functions (Configure File Accessing)
 */
/* ---- Add one entry to the schedule program (crond) ---- */
int SCHDAddSchedule(SchdInfo_t *info)
{
	int retcode;
	char user_buff[SCHD_CONF_LINE_LEN+1];
	char temp_buff[strlen(SCHD_CONF_TMPFILENAME)+5];
	FILE *cfgfile;		/* crontab */
	FILE *tmpfile;
	char srandom[5];	/* 4+1 */
	int  r_fd, w_fd;	/* over-write the original crontab file */
	ssize_t rsize;
	char buf[80];
				/* -- error check -- */
	retcode = SCHDCheckField(info);
	if (retcode == SCHD_ERROR) {
		return SCHD_EINVALID;
	}
				/* -- convert the format */
	memset(&user_buff[0], '\0', SCHD_CONF_LINE_LEN+1);
	retcode = SCHDConvtUserFormat(info, &user_buff[0], SCHD_CONF_LINE_LEN);
	if (retcode != SCHD_OK) {
		return retcode;
	}
				/* -- do job: add (access conf file) -- */
	cfgfile = fopen(SCHD_CONF_FILENAME, "r"); /* crontab */
				/* -- create crontab if it doesn't exist ---*/
	if (cfgfile == NULL) {
		cfgfile=fopen(SCHD_CONF_FILENAME, "w");
		if(cfgfile ==  NULL){
			return SCHD_EFILE;
		}
		memset(buf,'\0',sizeof(buf));
		sprintf(buf,"SHELL=/bin/sh\n");
		fwrite(buf,1, strlen(buf), cfgfile);
		sprintf(buf,"PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/bin\n");
		fwrite(buf,1, strlen(buf), cfgfile);
		sprintf(buf,"MAILTO=root\n");
		fwrite(buf,1, strlen(buf), cfgfile);
		sprintf(buf,"HOME=/\n");
		fwrite(buf,1, strlen(buf), cfgfile);
		sprintf(buf,"\n");
		fwrite(buf,1, strlen(buf), cfgfile);
		sprintf(buf,"# run-parts\n");
		fwrite(buf,1, strlen(buf), cfgfile);
	}
				/* prepare temp filename */
	memset(&srandom[0], '\0', 5);
	SCHDGetRandom4(&srandom[0]);
	memset(&temp_buff[0], '\0', strlen(SCHD_CONF_TMPFILENAME)+5);
	sprintf(&temp_buff[0], "%s%s", SCHD_CONF_TMPFILENAME, &srandom[0]);
	tmpfile = fopen(&temp_buff[0], "w+"); /* crontab.OOXX */
	if (tmpfile == NULL) {
		fclose(cfgfile);
		return SCHD_EFILE;
	}
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	while (fgets(rw_buffer, SCHD_MAX_BUFFER, cfgfile) != NULL) {
				/* compare the whole line */
		if (strcmp(rw_buffer, &user_buff[0]) == 0) {
			continue;
		} else {
			fputs(rw_buffer, tmpfile);
		} /* skip the same entry */
	}
				/* ADD (the last line) */
	fputs(&user_buff[0], tmpfile);
	fclose(cfgfile);
	fclose(tmpfile);
				/* write back */
	r_fd = open(&temp_buff[0], O_RDONLY);
	if (r_fd < 0) {
		return SCHD_ERROR;
	}
	w_fd = open(SCHD_CONF_FILENAME, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (w_fd < 0) {
		close(r_fd);
		return SCHD_ERROR;
	}
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	while((rsize = read(r_fd, &rw_buffer[0], (ssize_t)SCHD_MAX_BUFFER)) > 0) {
		write(w_fd, &rw_buffer[0], rsize);
		memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	}
	close(r_fd);
	close(w_fd);
	unlink(&temp_buff[0]);	/* remove temp file */
	return SCHD_OK;
}

/* ---- Del one entry to the schedule program (crond) ---- */
int SCHDDelSchedule(SchdInfo_t *info)
{
	int retcode;
	char user_buff[SCHD_CONF_LINE_LEN+1];
	char temp_buff[strlen(SCHD_CONF_TMPFILENAME)+5];
	FILE *cfgfile;		/* crontab */
	FILE *tmpfile;
	char srandom[5];	/* 4+1 */
	int  r_fd, w_fd;	/* over-write the original crontab file */
	ssize_t rsize;
				/* -- error check -- */
	retcode = SCHDCheckField(info);
	if (retcode == SCHD_ERROR) {
		return SCHD_EINVALID;
	}
				/* -- convert the format */
	memset(&user_buff[0], '\0', SCHD_CONF_LINE_LEN+1);
	retcode = SCHDConvtUserFormat(info, &user_buff[0], SCHD_CONF_LINE_LEN);
	if (retcode != SCHD_OK) {
		return retcode;
	}
				/* -- do job: del (access conf file) -- */
	cfgfile = fopen(SCHD_CONF_FILENAME, "r"); /* crontab */
	if (cfgfile == NULL) {
		return SCHD_EFILE;
	}
				/* prepare temp filename */
	memset(&srandom[0], '\0', 5);
	SCHDGetRandom4(&srandom[0]);
	memset(&temp_buff[0], '\0', strlen(SCHD_CONF_TMPFILENAME)+5);
	sprintf(&temp_buff[0], "%s%s", SCHD_CONF_TMPFILENAME, &srandom[0]);
	tmpfile = fopen(&temp_buff[0], "w+"); /* crontab.OOXX */
	if (tmpfile == NULL) {
		fclose(cfgfile);
		return SCHD_EFILE;
	}
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	while (fgets(rw_buffer, SCHD_MAX_BUFFER, cfgfile) != NULL) {
				/* compare the whole line */
		if (strcmp(rw_buffer, &user_buff[0]) == 0) {
			continue;
		} else {
			fputs(rw_buffer, tmpfile);
		} /* skip the same entry */
	}
				/* DEL (drop it) */
				/* fputs(user_buff, tmpfile); */
	fclose(cfgfile);
	fclose(tmpfile);
				/* write back */
	r_fd = open(&temp_buff[0], O_RDONLY);
	if (r_fd < 0) {
		return SCHD_ERROR;
	}
	w_fd = open(SCHD_CONF_FILENAME, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (w_fd < 0) {
		close(r_fd);
		return SCHD_ERROR;
	}
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	while((rsize = read(r_fd, &rw_buffer[0], (ssize_t)SCHD_MAX_BUFFER)) > 0) {
		write(w_fd, &rw_buffer[0], rsize);
		memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	}
	close(r_fd);
	close(w_fd);
	unlink(&temp_buff[0]);	/* remove temp file */
	return SCHD_OK;
}

/* Del "ALL" entries with the same "Command Line" argument to schedule 
 * program (crond). Only input "command" element is valid, the other will
 * discard.
 */
int SCHDDelScheduleByCmd(SchdInfo_t *info)
{
	int retcode;
	char *pstart;
	char user_buff[SCHD_CONF_LINE_LEN+1];
	char temp_buff[strlen(SCHD_CONF_TMPFILENAME)+5];
	FILE *cfgfile;		/* crontab */
	FILE *tmpfile;
	char srandom[5];	/* 4+1 */
	int  r_fd, w_fd;	/* over-write the original crontab file */
	ssize_t rsize;
				/* -- error check -- */
	retcode = SCHDCheckField(info);
	if (retcode == SCHD_ERROR) {
		return SCHD_EINVALID;
	}
				/* -- read the current conf file -- */
	cfgfile = fopen(SCHD_CONF_FILENAME, "r"); /* crontab */
	if (cfgfile == NULL) {
		return SCHD_EFILE;
	}
				/* prepare temp filename */
	memset(&srandom[0], '\0', 5);
	SCHDGetRandom4(&srandom[0]);
	memset(&temp_buff[0], '\0', strlen(SCHD_CONF_TMPFILENAME)+5);
	sprintf(&temp_buff[0], "%s%s", SCHD_CONF_TMPFILENAME, &srandom[0]);
	tmpfile = fopen(&temp_buff[0], "w+"); /* crontab.OOXX */
	if (tmpfile == NULL) {
		fclose(cfgfile);
		return SCHD_EFILE;
	}
	memset(&user_buff[0], '\0', SCHD_CONF_LINE_LEN+1);
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
				/* -- do job: compare all entries -- */
	while (fgets(rw_buffer, SCHD_MAX_BUFFER, cfgfile) != NULL) {
				/* get the command-line field */
				/* MM HH DD MM WW owner command-line */
				/*   1  2  3  4  5     6     <SPACE> */
		retcode = SCHDCutFields(rw_buffer, strlen(rw_buffer), 6, &user_buff[0]);
		if (retcode == SCHD_OK) {
				/* only check the Command Field */
			pstart = &user_buff[0];
			while((*pstart != ' ')&&(*pstart != '\0')&&(*pstart != '\n')) {
				pstart++;
			}
			*pstart = '\0';
			if (strcmp(&user_buff[0], info->command) == 0) {
				memset(&user_buff[0], '\0',  SCHD_CONF_LINE_LEN+1);
				continue;
			} /* DEL (drop it) */
			memset(&user_buff[0], '\0',  SCHD_CONF_LINE_LEN+1);
		}
		fputs(rw_buffer, tmpfile);
		memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	}			
	fclose(cfgfile);
	fclose(tmpfile);
				/* write back */
	r_fd = open(&temp_buff[0], O_RDONLY);
	if (r_fd < 0) {
		return SCHD_ERROR;
	}
	w_fd = open(SCHD_CONF_FILENAME, O_WRONLY|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
	if (w_fd < 0) {
		close(r_fd);
		return SCHD_ERROR;
	}
	memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	while((rsize = read(r_fd, &rw_buffer[0], (ssize_t)SCHD_MAX_BUFFER)) > 0) {
		write(w_fd, &rw_buffer[0], rsize);
		memset(&rw_buffer, '\0', SCHD_MAX_BUFFER);
	}
	close(r_fd);
	close(w_fd);
	unlink(&temp_buff[0]);	/* remove temp file */
	return SCHD_OK;
}

/*
 * Exported Functions (Daemon Accessing)
 */
/* ---- Start the schedule program (crond) ---- */
int SCHDStartSchedule(void)
{
	system(SCHD_EXE_SCHED_START);
	return SCHD_OK;
}

/* ---- Stop  the schedule program (crond) ---- */
int SCHDStopSchedule(void)
{
	system(SCHD_EXE_SCHED_STOP);
	return SCHD_OK;
}

#ifdef _TEST_PROG_
static  SchdInfo_t info1 = { "Don't Care Any Time Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1} }; /* the same as "info11" */
static  SchdInfo_t info2 = { "Do Care Time [Minute] Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,01,-1} };
static  SchdInfo_t info3 = { "Do Care Time [Minute:Hour] Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {4,0,23}, {5,0,59} };
static  SchdInfo_t info4 = { "Do Care Time [Minute:Hour:Week] Fields!", {-1,1,6}, {-1,-1,-1}, {-1,-1,-1}, {-1,9,18}, {-1,15,45} };
static  SchdInfo_t info5 = { "Do Care All Time Fields!", {-1,0,6}, {1,2,-1}, {4,5,6}, {-1,-1,-1}, {-1,8,58} }; /* the same as "info15" */

static  SchdInfo_t info11 = { "Don't Care Any Time Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1} }; /* the same as "info1" */
static  SchdInfo_t info12 = { "Do Care Time [Minute] Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {-1,2,50} };
static  SchdInfo_t info13 = { "Do Care Time [Minute:Hour] Fields!", {-1,-1,-1}, {-1,-1,-1}, {-1,-1,-1}, {4,8,16}, {15,0,59} };
static  SchdInfo_t info14 = { "Do Care Time [Minute:Hour:Week] Fields!", {2,1,6}, {-1,-1,-1}, {-1,-1,-1}, {-1,9,-1}, {-1,15,-1} };
static  SchdInfo_t info15 = { "Do Care All Time Fields!", {-1,0,6}, {1,2,-1}, {4,5,6}, {-1,-1,-1}, {-1,8,58} };	/* the same as "info5" */

void usage(char *cmd)
{
	printf("Usage:\t%s [add/del/del-all]\n", cmd);
}

int main(int argc, char **argv)
{
	if (argc < 2) {
		usage(argv[0]);
	} else if(strcmp(argv[1], "add") == 0) {
		SCHDAddSchedule(&info1);
		SCHDAddSchedule(&info2);
		SCHDAddSchedule(&info3);
		SCHDAddSchedule(&info4);
		SCHDAddSchedule(&info5);
		SCHDAddSchedule(&info11);
		SCHDAddSchedule(&info12);
		SCHDAddSchedule(&info13);
		SCHDAddSchedule(&info14);
		SCHDAddSchedule(&info15);
	} else if (strcmp(argv[1], "del") == 0) {
		SCHDDelSchedule(&info11);
		SCHDDelSchedule(&info12);
		SCHDDelSchedule(&info13);
		SCHDDelSchedule(&info14);
		SCHDDelSchedule(&info15);
	} else if (strcmp(argv[1], "del-all") == 0) {
		SCHDDelScheduleByCmd(&info1);
		SCHDDelScheduleByCmd(&info15);
	}
	return 0;
}
#endif /* _TEST_PROG_ */
