
/***************************************************************
 * (C) COPYRIGHT SerComm Corporation All Rights Reserved, 2004.
 *
 *      Use of this software is restricted to the terms and
 *      conditions of SerComm's software license agreement.
 *
 *                        www.sercomm.com
 ****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include <assert.h>
#include "ddns_log.h"

#define INDEX_LEN	5
#define MAX_LEN   	256
#define MAX_KEY_LEN   	64
#define MSG_LEN 	108
#define TIME_LEN    	20
#define MAX_MSG_LEN 	(MSG_LEN+TIME_LEN+2)
#define MAX_MSGF_LEN 	(MAX_MSG_LEN*100+1)
#define SYS_CONF "/etc/system.conf"
#define LOGFILE	 "/etc/logfile"
#define SYSLOG_SEC   "[LOG]"

static int LOG_2sect(char *sect, FILE *fp)
{
   char buf[MAX_LEN],sec[MAX_KEY_LEN],*p;
   int  ln;

   p = strstr(sect,"\n");
   if (p)
      *p = 0;
   if (sect[0] != '[')
   {
      sprintf(sec,"[%s]",sect);
      p = sec;
   }
   else
      p = sect;
   rewind(fp);
   ln = strlen(p);
   while(fgets(buf,sizeof(buf),fp))
   {
      if (buf[0] != '[')
         continue;
      if (strncmp(buf,p,ln)==0)
         return 0;
   }
   return 1;
}

static int LOG_GetStr(char *sect, char *key, char *val, int size, FILE* fp)
{
   int  i,bl;
   char *buf,*pf=NULL,*p,*p0;
   char b[MAX_LEN]; /* a line read from ini file */

   if (val==NULL || fp==NULL || key==NULL)
      return -1;

   if (sect) {
      if (LOG_2sect(sect,fp) != L_OK) // section not found
         return -1;
   } else
      rewind(fp);

   if (size > MAX_LEN) {
      bl = size+MAX_KEY_LEN;
      pf = buf = malloc(bl);
      if (!buf)
         return -1;
   } else {
      bl = sizeof(b);
      buf = b;   
   }

   i = strlen(key);
   while(fgets(buf,bl,fp))
   {
      if (buf[0]=='[')
         break; // end of section
      p=buf;
      while(*p==' ')
         p++;
      if (strncmp(p,key,i)==0)
      {
         p0=p+i;
         while (*p0==' ')
            p0++;
         if (*p0 != '=')
            continue;
         p0++;
         while (*p0==' ')
            p0++;
         p=buf+strlen(buf)-1;
         while(*p==0x0d || *p==0x0a)
            p--;
         *(p+1)=0;
         if (strlen(p0) > size)
            *(p0+size-1)=0;
         strcpy(val,p0);
	 free(pf);
         return L_OK;
      } //if (strncmp())
   } //while (fgets())
   free(pf);
   return L_ERROR;
}

static int LOG_GetInt(char *sect, char *key, int *val, FILE *fp)
{
   char buf[MAX_KEY_LEN];
   int ret;

   ret = LOG_GetStr(sect,key,buf,sizeof(buf),fp);
   if (ret==L_OK)
      *val = atoi(buf);

   return ret;
}

int AddDDNSLog(char *task,int level, int index,...)
{
   char ikey[INDEX_LEN];
   char buf[MAX_MSG_LEN];
   char tmsg[MSG_LEN+2]={0},msg[MSG_LEN+2]={0};
   char dt[TIME_LEN];
   double dtime;
   int  i,fd,ret;
   time_t t;
   struct tm *ptm;
   char TMPF[] = "/tmp/LCSXXXXXX";
   FILE *fp;
   struct stat s;
   va_list ap;

   if (level > SYSLOG_ALL || level < SYSLOG_ERROR)
      return L_ERROR; // out of range

   fp = fopen(LOGFILE,"rt");
   if(fp==NULL){
	return L_ERROR;
   }

   sprintf(ikey ,"%d",index);
   
   ret=LOG_GetStr(task,ikey,tmsg, MSG_LEN+1,fp);

   if(ret != L_OK){
	fclose(fp);
	return L_ERROR;
   }	
   fclose(fp);

   va_start(ap,index);
   vsprintf(msg,tmsg,ap); 	
   va_end(ap);

   msg[MSG_LEN] = 0;

   time(&t);
   ptm = localtime(&t);
   strftime(dt,TIME_LEN,"%m/%d/%Y %T",ptm);

   sprintf(buf,"%s %s",dt,msg);

   buf[MAX_MSG_LEN-2] = 0;
   if (strstr(buf,"\n") == NULL)
   {  // the message does not contain a '\n', append one
      i = strlen(buf);
	buf[i] = '\n';
	buf[i+1] = 0 ;
   }

   fp = fopen(SYS_CONF,"r");

   if (fp)
   {
      LOG_GetInt(SYSLOG_SEC,"log",&i,fp);
      if((i > SYSLOG_ALL) || (i < SYSLOG_ERROR))
		i = SYSLOG_ALL;
      fclose(fp);
   }
   else
      i = SYSLOG_NORMAL; // default


   if ( level > i) // log disable 
      return L_OK;

   if (stat(TMPF,&s) == 0)
   {  
	for(i=0;i<30;i++){
   		time(&t);	
		dtime=difftime(t,s.st_atime);  

		if((dtime > 60) || (dtime < (-60)) ) 
			break;

		sleep(2);

		if(stat(TMPF,&s) == 0)
			continue;
		else
			break;
    	}

   }

   fd = open(TMPF,O_CREAT|O_WRONLY|O_TRUNC,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
   if (fd == -1){
      return L_ERROR;
   }
   else
   {
      write(fd,buf,strlen(buf));
      close(fd);
   }

   if (stat(DDNSLOG_FILE,&s) == 0)
   {  // LOG_FILE exists
      sprintf(buf,"/usr/bin/head -n 99 %s >> %s 2> /dev/null",DDNSLOG_FILE,TMPF);
      system(buf);
   }

   sprintf(buf,"/bin/mv -f %s %s > /dev/null 2> /dev/null",TMPF,DDNSLOG_FILE);
   system(buf);

   // truncate to MAX_MSGF_LEN
   if ((stat(DDNSLOG_FILE,&s) == 0) && s.st_size > MAX_MSGF_LEN)
      truncate(DDNSLOG_FILE,MAX_MSGF_LEN);

   return L_OK;
}

int  ClearDDNSLog(void)
{
   FILE *fp;

   fp = fopen(DDNSLOG_FILE,"w");
   if (fp==NULL)
      return L_ERROR;
   fclose(fp);
   return L_OK;
}

