/*
 *             SerComm CORPORATION PROPRIETARY INFORMATION         
 *                                                                    	
 *     This software is supplied under the terms of a license agree-   	
 *     ment or non-disclosure agreement with SerComm Corporation         
 *     and may not be copied nor disclosed except in accordance       
 *     with the terms of that agreement.                              
 *                                                                   
 */

#ifndef __SC_DEBUG_C__
#define __SC_DEBUG_C__

#include "sc_config.h"
#include "sc_debug.h"

#ifdef _SC_DBG_ON_ALL_

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <features.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/utsname.h>
#include <net/if.h>
#include <net/if_arp.h>
#ifdef __GLIBC__
#include <net/if_packet.h>
#else
#include <linux/if_packet.h>
#endif
#include <netdb.h>
#include <linux/if_ether.h>

/* Debug main function */

#define SC_DBG_PKT_MSG_LEN		512

typedef struct sc_dbg_pkt
{
	unsigned char dst_mac[6];
	unsigned char src_mac[6];
	unsigned short sap;
	char info[22];
	char data[SC_DBG_PKT_MSG_LEN];
} SC_DBG_PACKET;

int sc_pkt(const char *format, ...)
{
	//int ret = 0;
	va_list ap;
	SC_DBG_PACKET dbg_pkt;
	int sc_dbg_fd = -1;
	struct sockaddr sll;

	if (-1 == (sc_dbg_fd = socket(AF_PACKET, SOCK_PACKET, htons(ETH_P_ALL))))
	{
		//printf("create socket error\n");
		return -1;
	}	
	memcpy(dbg_pkt.dst_mac, "\xff\xff\xff\xff\xff\xff", 6);
	memcpy(dbg_pkt.src_mac, "\x99\x99\x99\x99\x99\x99", 6);	// Here should get local MAC...
	dbg_pkt.sap = 0x9999;

	memset(&sll, 0, sizeof(sll));
	//sll.spkt_family = AF_PACKET;
	//sll.spkt_protocol = htons(ETN_P_ALL);
	strcpy(sll.sa_data, "eth0");	

	memset(dbg_pkt.data, 0, SC_DBG_PKT_MSG_LEN);
	va_start(ap, format);
	vsnprintf(dbg_pkt.data, SC_DBG_PKT_MSG_LEN, format, ap);
	va_end(ap);
	sprintf(dbg_pkt.info, "[%s %s]", __DATE__, __TIME__);
	sendto(sc_dbg_fd, &dbg_pkt, sizeof(SC_DBG_PACKET), 
				0, (struct sockaddr *)&sll, sizeof(struct sockaddr));
	//printf("sendto ret=%d\n", ret);
	close(sc_dbg_fd);
	return 0;
}

#define SC_DBG_MAXSIZE			102400
int sc_printf(char *filename, const char *format, ...)
{
	va_list ap;
	FILE *fp;
	struct stat st;
	int ret;

#if 0	
	struct timeval nnn;
#endif

	if (NULL == filename)
	{
		return -1;
	}
	ret=stat(filename, &st);
	if (!ret && st.st_size>0 && st.st_size>SC_DBG_MAXSIZE)
	{
		fp = fopen(filename, "w");		
	}
	else
	{
		fp = fopen(filename, "a");
	}
	if (NULL == fp)
	{
		return -1;
	}
#if 0	
	gettimeofday(&nnn, NULL);
	fprintf(fp, "[%lu] ", ((nnn.tv_sec*1000000+nnn.tv_usec)/1000));
#else
	fprintf(fp, "[%s %s] ", __DATE__, __TIME__);
#endif
	va_start(ap,format);
	vfprintf(fp,format,ap);
	va_end(ap);
	fclose(fp);
	return 0;	
}
/* Debug main function end */

#endif


#endif

