/***************************************************************
*             Copyright (C) 2006 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "PRO_file.h"		//function call for configuration
#include "user_config.h"		//HLD
//#include "user_internal.h"
#include "sc_debug.h"
#include "conf_sec.h"

#define CTRL_STR_LEN	50
#define AUTH_COMMAND	"/usr/local/bin/auth >/dev/null 2>/dev/null"

#define READ		1
#define WRITE		0


char str1[UserLen+1];
char str2[PassLen+1];
//-----------------------internal function-----------------------

static int parse_str(char input[],char *name,char *pw)	// xxx/yyy -> str1=xxx, str2=yyy
{
	int name_len=0,pw_len=0;
	char *p=input;

    if (NULL == p || NULL == name || NULL == pw)
    {
    	return -1;
    }
	//If input is empty,we think name is empty and pw is empty,so we must set name and pw to empty.
	if(*p == '\0')
	{
		name[0]=0;
		pw[0]=0;
		return 0;
	}
	while(*p!= UserItemSeparator)
	{
		name_len++;
		p++;
	}
    pw_len = strlen(++p);
    if (pw_len > PassLen)
    {
    	pw_len = PassLen;
    }
    if (name_len > UserLen )
    {
		name_len = UserLen;
    }
	strncpy(name,input,name_len);
    name[name_len]= '\0';
	strncpy(pw,p,pw_len);
    pw[pw_len]= '\0';
	return 0;
}

/* return user numbers and not include detail check */
#if 0
static int check_user(void)
{
	FILE *fd;
	int num=0;
	int ret;
 
	fd = fopen(SYSTEM_CONF,"rt");
	if(!fd)
		return -1;
	
	ret=PRO_GetInt(SEC_USER, USER_CURUSER, &num, fd);	
	if(ret)
	{
		fclose(fd);
		return -2;	
	}

	fclose(fd);
	return (num);
}
#endif
#ifdef _ADD_ONE_USER_AT_A_TIME_
/* return user location i else return -1 : open file error
 *             			      -2 : search no user 
 */
static int search_user(char *oldname)	
{
	int i,ret;
	FILE *fd;
	char u_item[UserItemLen+1],strname[AllLen+2];
	char name[UserLen+1],passwd[PassLen+1];

	fd = fopen(SYSTEM_CONF,"rt");
	if(!fd)
	{
		return -1;
	}		
	for(i=USER_USER_START_NUM; i <= USER_MAX_USER_NUM; i++)
	{
		sprintf(u_item, USER_USER_FORMAT, i);	
		ret=PRO_GetStr(SEC_USER,u_item,strname,AllLen+2,fd);	
		if(ret)
		{
			fclose(fd);
			return -3;
		}
		parse_str(strname,name,passwd);
		if(!strcmp(oldname,str1))	
		{
			break;
		}
		if(i==USER_MAX_USER_NUM)
		{
			fclose(fd);
			return -2;
		}
	}
	fclose(fd);
	return i;
}
#endif

static int check_range(char *p)
{
	while(*p)
	{
		if( (*p>=0x41 && *p<=0x5a) || (*p>=0x61 && *p<=0x7a) || (*p>=0x30 && *p<=0x39) || (*p==0x5F))
		{
			p++;
		}
		else
		{
			return 1;		// invalid string
		}
	}	
	return 0;
}
#ifndef _SC_ONLY_ONE_USER_
static 
#endif
int check_data(char *name, char *pass)
{
	if(check_range(name))
	{
		return 1;
	}
	if(check_range(pass))
	{
		return 2;
	}
	return 0;
}
int check_user_data(char *user_item,char *name,char *pass)
{

	if(NULL == user_item || NULL == name || NULL == pass)
	{
		return 1;
	}
	memset(name,'\0',strlen(name));
	memset(pass,'\0',strlen(pass));
	if(parse_str(user_item, name, pass))
	{
		return 1;
	}
	if(check_data(name, pass))
	{
		return 1;
	}
	return 0;
}

//------------------------external function call--------------------



int ReadUserConf(USER_CONF *pconf)
{
	FILE *fd;
	int ret;

	if(!pconf)
		return User_InputParamErr;	
	fd = fopen(SYSTEM_CONF,"rt");
	if(!fd)
	{
		return User_ReadUserConfErr;
	}

	ret=PRO_GetInt(SEC_USER, USER_LOGIN_CHECK, &pconf->enable, fd);	
	if(ret)
     {
		fclose(fd);
		return User_ReadUserConfErr;
	}

	ret=PRO_GetInt(SEC_MAN, MAN_MAX_USER, &pconf->max_user, fd);
	if(ret)
     {
		fclose(fd);
		return User_ReadUserConfErr;
	}
#if 0
	ret=PRO_GetInt(SEC_USER, USER_CURUSER, &pconf->cur_user, fd);	
	if(ret)
     {
		fclose(fd);
		return User_ReadUserConfErr;
	}
#endif
	ret=ReadUsers(0,USER_MAX_USER_NUM, pconf->user_list);
    if(ret)
    {
		return User_ReadUserConfErr;
	}
	fclose(fd);
	return User_OK;
}


int SaveUserConf(USER_CONF *uinfo)
{
	int	ret=0;
	int	i=0;
	char user_item[UserItemLen+1];
	char user_content[AllLen+2]; //length= username_len + passwd_len + '/'+'\0'
 
	ret=PRO_SetInt(SEC_USER, USER_LOGIN_CHECK, uinfo->enable, SYSTEM_CONF);
	if(ret)
	{
		return User_SaveUserConfErr;
	}
	//check user list be puted in set function.
	for(i=0; i < USER_MAX_USER_NUM; i++)
	{
		sprintf(user_item,USER_USER_FORMAT,i+1);	
		if (uinfo->user_list[i].name[0])
		{
			sprintf(user_content,"%s%c%s",uinfo->user_list[i].name,UserItemSeparator,uinfo->user_list[i].password);
		}
		else
		{
            user_content[0]=0;		
		}
		ret=PRO_SetStr(SEC_USER, user_item, user_content, SYSTEM_CONF);
		if(ret)
		{
     			return User_SaveUserConfErr;
		}
	}
#if 0
	ret=PRO_SetInt(SEC_USER, USER_CURUSER, uinfo->cur_user, SYSTEM_CONF);
	if(ret)
	{
		return User_SaveUserConfErr;
	}
#endif
	system("/usr/local/bin/auth -user >/dev/null 2>/dev/null");	/* update htpasswd */
	return User_OK;
}



/* return user location i else return -1 : open file error
 *             			      -2 : search no user 
 */

int ReadUsers(int start,int num,USER_CONF1 user[])
{
	FILE *fd;
	int i;
	char user_item[UserItemLen+1];		//ex: user0,user1,user2
	char str[AllLen+2];			//"name/password",length must be AllLen+2,end is '\0'
	int ret;
	
	if(start<0 || num<0)
	{
		return User_InputParamErr;	
	}	
	fd = fopen(SYSTEM_CONF,"rt");
	if(!fd)
	{
		return User_ReadUsersErr;
	}
	if (start > USER_MAX_USER_NUM -1 )//start range:0~19 ,so it must -1.
	{
		start = USER_MAX_USER_NUM -1;
	}
	if (num>USER_MAX_USER_NUM - start)
	{
		num = USER_MAX_USER_NUM -start;
	}
	if (num > USER_MAX_USER_NUM)
	{
		num = USER_MAX_USER_NUM;
	}
	for(i=start;i<num;i++)		//strat == 0 ,"num" is user number that need to get from "start".
	{
		sprintf(user_item, USER_USER_FORMAT, i+1);
		ret=PRO_GetStr(SEC_USER, user_item, str, AllLen+2, fd);
		if(ret)
		{
			fclose(fd);
			return User_ReadUsersErr;
		}
		if(parse_str(str,user[i-start].name,user[i-start].password))
		{
			return User_ReadUsersErr;
		}
		//SC_DBG_CGI("user[i-start].name=%s\n",user[i-start].name);
		/*
        if(0 == strlen(user[i-start].name))	//if username is empty,we think have read all user.
        {
        	break;
        }
        */
	}
	fclose(fd);
	return User_OK;
	}

int ReadAdminConf(ADMIN_CONF *pconf)
{
	FILE *fd;
	int ret;

	if(!pconf)
		return User_InputParamErr;	
	fd = fopen(SYSTEM_CONF,"rt");
	if(!fd)
	{
		return User_ReadAdminConfErr;
	}

	ret=PRO_GetStr(SEC_USER, USER_ADMIN_NAME, pconf->adminname, UserLen+1, fd);	
	if(ret)
	{
		fclose(fd);
		return User_ReadAdminConfErr;
	}
	ret=PRO_GetStr(SEC_USER, USER_ADMIN_PASSWORD, pconf->adminpw, PassLen+1, fd);
	if(ret)
	{
		fclose(fd);
		return User_ReadAdminConfErr;
	}
#ifdef _SC_WEB_SERVER_ADM_TIMEOUT_
	ret=PRO_GetInt(SEC_USER, USER_ADMIN_TIMEOUT, &pconf->adm_timeout, fd);
	if(ret)
	{
		fclose(fd);
		return User_ReadAdminConfErr;
	}
#endif
	fclose(fd);
	return User_OK;
}

int SaveAdminConf(ADMIN_CONF *pconf)
{
	int	ret;
	
	if(!pconf)
	{
		return User_InputParamErr;	
	}
	if(check_data(pconf->adminname, pconf->adminpw))
	{
		return User_InputParamErr;
	}


	ret=PRO_SetStr(SEC_USER, USER_ADMIN_NAME, pconf->adminname, SYSTEM_CONF);
	if(ret)
	{
     	return User_SaveAdminConfErr;
	}

	ret=PRO_SetStr(SEC_USER, USER_ADMIN_PASSWORD, pconf->adminpw, SYSTEM_CONF);
	if(ret)
	{
     	return User_SaveAdminConfErr;
	}
#ifdef _SC_WEB_SERVER_ADM_TIMEOUT_
	ret=PRO_SetInt(SEC_USER, USER_ADMIN_TIMEOUT, pconf->adm_timeout, SYSTEM_CONF);
	if(ret)
	{
     	return User_SaveAdminConfErr;
	}
#endif
	
	system("/usr/local/bin/auth -adm >/dev/null 2>/dev/null");	/* update htpasswd */
	return User_OK;
}


#ifdef _ADD_ONE_USER_AT_A_TIME_
int ModifyUser(char old[UserLen+1],USER_CONF1 uinfo)
{
	int ret;
	int i;
	char user_item[UserItemLen+1];
	char str[AllLen+2];		//"name/password"

	//search name & passwd from "old"
	i=search_user(old);
	if(i<0){
		DBG( sprintf(ulogmsg, "search old user %s, ret=%d\n",old,i); )
		DBG( uLogMsg(ulogmsg); )
		return User_InputParamErr;	
	}

	if(check_data(uinfo.name, uinfo.password)){
		DBG( uLogMsg("Not a valid user\n"); )	
		return User_InputParamErr;
	}

	// can't the same with other old users' name
	if( strcmp(uinfo.name, old) && search_user(uinfo.name) != -2 )
		return User_InputParamErr;	
	

#ifdef ALLOW_SPACE
	strcpy(uinfo.name, newname);
	strcpy(uinfo.password, newpass);
#endif
	sprintf(user_item, USER_USER_FORMAT, i);	
	sprintf(str,"%s%c%s",uinfo.name,'/',uinfo.password);

	ret=PRO_SetStr(SEC_USER, user_item, str, SYSTEM_CONF);
	if(ret)
		return User_ModifyUserErr;
	system(AUTH_COMMAND);	/* update htpasswd */
	return User_OK;
}

int AddUser(USER_CONF1 uinfo)
{
	int ret;
	char user_item[UserItemLen+1];
	char str[AllLen+2];		//"name/password"
	int num;	
	int ccode;

//	if(check_data(uinfo.name, uinfo.password))
	ccode=check_data(uinfo.name, uinfo.password);
	if(ccode)
		return User_InputParamErr;

#ifdef ALLOW_SPACE
	strcpy(uinfo.name, newname);
	strcpy(uinfo.password, newpass);
#endif

#ifdef _SC_ONLY_ONE_USER_
	if(search_user(uinfo.name) >= 0)	//make sure new user not exist
#else
	if(search_user(uinfo.name) != -2)	//make sure new user not exist
#endif	
		return User_AddUserDupErr;
	num = check_user();
	if(num >= USER_MAX_USER_NUM)
		return User_AddUserOverErr;	//over maximum user
	if(num < 0)
		return User_AddUserErr;
	
	sprintf(user_item,USER_USER_FORMAT,num+1);	
	sprintf(str,"%s%c%s",uinfo.name,'/',uinfo.password);

	ret=PRO_SetStr(SEC_USER, user_item, str, SYSTEM_CONF);
	if(ret)
		return User_AddUserErr;

	system(AUTH_COMMAND);	/* update htpasswd */


	return User_OK;	
}

int DelUser(char name[UserLen+1])
{
	int i,ret,num;
	char user_item[128];

	num = check_user();
	if(num < 0){
		DBG( sprintf(ulogmsg, "check user ret=%d\n", ret); )
		DBG( uLogMsg(ulogmsg); )
		return User_DelUserErr;
	}

	i = search_user(name);		//i: user location
	if(i<0){
		DBG( sprintf(ulogmsg, "search user %s ,ret=%d\n", name, ret); )
		DBG( uLogMsg(ulogmsg); )
		return User_InputParamErr;	
	}

	/* adjust user loaction */
	sprintf(user_item,USER_USER_FORMAT,i);	
	ret=PRO_SetStr(SEC_USER, user_item, "", SYSTEM_CONF);
	if(ret)
		return User_DelUserErr;


	#if 0
	ret=adjust_user(&i, &num);
	if(ret){
		DBG( sprintf(ulogmsg, "adjust user ret=%d\n", ret); )
		DBG( uLogMsg(ulogmsg); )	
		return User_DelUserErr;
	}	
	#endif
		
		
	system(AUTH_COMMAND);	/* update htpasswd */
	return User_OK;
}

int DelAllUsers(void)
{
    int i,ret;
    char user_item[UserItemLen+1];

    for(i=USER_USER_START_NUM; i<=USER_MAX_USER_NUM; i++) {
		sprintf(user_item,USER_USER_FORMAT,i);	
		ret=PRO_SetStr(SEC_USER, user_item, "", SYSTEM_CONF);
		if(ret)
			return User_DelAllUsersErr;
    }

    // Set user control permission to zero
    ret=PRO_SetInt(SEC_USER, USER_PT_CTRL, 0, SYSTEM_CONF);
    if(ret)
		return User_DelAllUsersErr;

    system(AUTH_COMMAND);	/* update htpasswd */
    return User_OK;
}
#endif	// _ADD_ONE_USER_AT_A_TIME_

#ifdef _HAS_VIEWER_ACCOUNT_
int ReadViewerInfo(VIEWER_INFO* vinfo)
{
	FILE* fp;
	int ret;
	
	fp = fopen(SYSTEM_CONF, "r+");
	ret = PRO_GetStr(SEC_USER, USER_VIEWER_NAME, vinfo->name, 
			 UserLen+1, fp);
	if(ret < 0)
		return User_ReadUserConfErr;

	ret = PRO_GetStr(SEC_USER, USER_VIEWER_PASSWORD, vinfo->pswd, 
			 PassLen+1, fp);
	if(ret < 0)
		return User_ReadUserConfErr;
	
	fclose(fp);

	return User_OK;
}

int SaveViewerInfo(VIEWER_INFO* vinfo)
{
	int ret;

	ret = PRO_SetStr(SEC_USER, USER_VIEWER_NAME, vinfo->name, SYSTEM_CONF);
	if(ret)
		return User_SaveUserConfErr;	

	ret = PRO_SetStr(SEC_USER, USER_VIEWER_PASSWORD, vinfo->pswd, SYSTEM_CONF);
	if(ret)
		return User_SaveUserConfErr;	
	
	return User_OK;
}
#endif

int perm_match(int read, char* key, int* perm_ptr)
{
	FILE* fp;
	int i, tmp=0;
	char cfgstr[2][CTRL_STR_LEN]={{0},{0}};
//	char str[CTRL_STR_LEN]={0};
	char* p;
	
	if(read)
	{
		*perm_ptr = 0;
		fp = fopen(SYSTEM_CONF, "r+");
		PRO_GetStr(SEC_USER, key, cfgstr[0], CTRL_STR_LEN, fp);
		
		for(i=0; i<USER_MAX_USER_NUM; i++)
		{
			if(i==USER_MAX_USER_NUM-1)
		{
				sscanf(cfgstr[i%2], "%d", &tmp);
			}			
			else
			{
				sscanf(cfgstr[i%2], "%d,%s", &tmp, cfgstr[(i+1)%2]);
			}
			*perm_ptr |= (tmp<<i);
		}
		fclose(fp);
	}
	else	// write
	{
		p = cfgstr[0];
		for(i=0; i<USER_MAX_USER_NUM; i++)
		{
			tmp = (*perm_ptr & (1<<i))>>i;
			if(i==USER_MAX_USER_NUM-1)
			{
				sprintf(p, "%d", tmp);
			}				
			else
			{
				sprintf(p, "%d,", tmp);
				p += 2;
			}
		}
		PRO_SetStr(SEC_USER, key, cfgstr[0], SYSTEM_CONF);

		system(AUTH_COMMAND);
	}
	return 0;
}


int ReadAllUserPTPerm(int *perm)
{
	int ccode;

	ccode = perm_match(READ, USER_PT_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}

int SetAllUserPTPerm(int *perm)
{
	int ccode;

	ccode = perm_match(WRITE, USER_PT_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}


int ReadAllUserAdminPerm(int *perm)
{
	int ccode;

	ccode = perm_match(READ, USER_ADM_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}

int SetAllUserAdminPerm(int *perm)
{
	int ccode;

	ccode = perm_match(WRITE, USER_ADM_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}


int ReadAllUserIOPerm(int *perm)
{
	int ccode;

	ccode = perm_match(READ, USER_IO_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}

int SetAllUserIOPerm(int *perm)
{
	int ccode;

	ccode = perm_match(WRITE, USER_IO_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;
	
	return User_OK;
}


int ReadAllUserAudioInPerm(int *perm)
{
	int ccode;

	ccode = perm_match(READ, USER_AUDIO_IN_CTRL, perm);
	if(ccode)
	{
		return User_ReadUserConfErr;
	}

	return User_OK;
}

int SetAllUserAudioInPerm(int *perm)
{
	int ccode;

	ccode = perm_match(WRITE, USER_AUDIO_IN_CTRL, perm);
	if(ccode)
	{
		return User_ReadUserConfErr;
	}
	
	return User_OK;
}


int ReadAllUserAudioOutPerm(int *perm)
{
	int ccode;

	ccode = perm_match(READ, USER_AUDIO_OUT_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}

int SetAllUserAudioOutPerm(int *perm)
{
	int ccode;

	ccode = perm_match(WRITE, USER_AUDIO_OUT_CTRL, perm);
	if(ccode)
		return User_ReadUserConfErr;

	return User_OK;
}


int USER_RdAllCfg(USER_CFG* user_cfg)
{
	int ret;

	ret = ReadAdminConf(&(user_cfg->adminfo));
	if(ret < 0)
		return ret;

#ifdef _HAS_VIEWER_ACCOUNT_	
	ret = ReadViewerInfo(&(user_cfg->vinfo));
	if(ret < 0)
		return ret;
#endif

//	ret = ReadUsers(0, MaxUserNum, user_cfg->uinfo);
	ret = ReadUserConf(&(user_cfg->uinfo));
	if(ret < 0)
		return ret;

	ret = perm_match(READ, USER_AUDIO_IN_CTRL, &user_cfg->audio_in_ctrl);
	if(ret < 0)
		return ret;
	
	ret = perm_match(READ, USER_AUDIO_OUT_CTRL, &user_cfg->audio_out_ctrl);
	if(ret < 0)
		return ret;

	ret = perm_match(READ, USER_IO_CTRL, &user_cfg->io_ctrl);
	if(ret < 0)
		return ret;

	ret = perm_match(READ, USER_ADM_CTRL, &user_cfg->adm_ctrl);
	if(ret < 0)
		return ret;

	return User_OK;
}

int USER_WrAllCfg(USER_CFG* user_cfg)
{
	int ret;

	ret = SaveAdminConf(&(user_cfg->adminfo));
	if(ret < 0)
		return ret;

#ifdef _HAS_VIEWER_ACCOUNT_
	ret = SaveViewerInfo(&(user_cfg->vinfo));
	if(ret < 0)
		return ret;
#endif	
        ret=SaveUserConf(&(user_cfg->uinfo));
	if(ret < 0)
		return ret;

	ret = perm_match(WRITE, USER_AUDIO_IN_CTRL, &user_cfg->audio_in_ctrl);
	if(ret < 0)
		return ret;
	
	ret = perm_match(WRITE, USER_AUDIO_OUT_CTRL, &user_cfg->audio_out_ctrl);
	if(ret < 0)
		return ret;

	ret = perm_match(WRITE, USER_IO_CTRL, &user_cfg->io_ctrl);
	if(ret < 0)
		return ret;

#if 0
	ret = perm_match(WRITE, USER_ADM_CTRL, &user_cfg->adm_ctrl);
	if(ret < 0)
		return ret;
#endif

	return User_OK;
}

/*
 * -1 		error
 *	0		okey
 *	other	dup
 */
int user_dup_check(USER_CFG	*user_conf)
{
	int i, j; //, ccode;

	//ccode = 0;
	if (NULL == user_conf)
	{
		return -1;
	}
    for(i=0;i<USER_MAX_USER_NUM;i++)
    {
        if (0 == user_conf->uinfo.user_list[i].name[0])
        {
            continue;
        }
//        ccode++;
        for (j=0;j<USER_MAX_USER_NUM;j++)
        {
	        if (0 == user_conf->uinfo.user_list[j].name[0])
	        {
	            continue;
	        }
			
        	if((j !=i && !strncmp(user_conf->uinfo.user_list[i].name,user_conf->uinfo.user_list[j].name,UserLen))
        	    || !strncmp(user_conf->uinfo.user_list[i].name,user_conf->adminfo.adminname,UserLen))
			{
				return 1;
			}
		}
/*
		if (ccode >= user_conf->uinfo.cur_user)
		{
		    break;
		}
*/		
    }
	return User_OK;
}

#ifdef _USER_TEST_
int main(int argc, char** argv)
{
	int i, perm[USER_MAX_USER_NUM]={0};
	USER_CFG cfg;

	if(atoi(argv[1])==1)
	{
		memset(&cfg, 0, sizeof(cfg));
		sprintf(cfg.adminfo.adminname, "adm");
		sprintf(cfg.adminfo.adminpw, "admpw");
	
		sprintf(cfg.uinfo[1].name, "newuser");
		sprintf(cfg.uinfo[1].password, "newpw");
		
		cfg.audio_in_ctrl = 0x0f0f0;
		cfg.audio_out_ctrl = 0xf0f0f;
		cfg.io_ctrl = 0xccccc;
		cfg.adm_ctrl = 0xffc00;

		USER_WrAllCfg(&cfg);
	}
	else if(atoi(argv[1])==2)
	{
		USER_RdAllCfg(&cfg);
		printf("admin: %s, %s\n", cfg.adminfo.adminname, cfg.adminfo.adminpw);
		printf("user2: %s, %s\n", cfg.uinfo[1].name, cfg.uinfo[1].password);
		printf("ctrl: %x\n", cfg.audio_in_ctrl);
		printf("ctrl: %x, %x, %x, %x\n", cfg.audio_in_ctrl, 
			cfg.audio_out_ctrl, cfg.io_ctrl, cfg.adm_ctrl);
	}
	return 0;
}
#endif

