/***************************************************************
*             Copyright (C) 2006 by SerComm Corp.
*                    All Rights Reserved.
*
*      Use of this software is restricted to the terms and
*      conditions of SerComm's software license agreement.
*
*                        www.sercomm.com
****************************************************************/
#ifndef _USER_CONFIG_H_
#define _USER_CONFIG_H_

#include "sc_config.h"

#ifdef _OEM_0001_0000_
#define USER_MAX_USER_NUM	4
#else
#define USER_MAX_USER_NUM	20
#endif
#define USER_USER_START_NUM	1
//#define USER_USER_END_NUM	USER_MAX_USER_NUM
//#define	MaxUserNum	USER_MAX_USER_NUM

#define		UserLen		32		//8 
#define		PassLen		64
#define		AllLen		UserLen+PassLen
#define		UserItemLen	6		//"userXX"

#define		yes		1
#define		no		0



#define		UserItemSeparator	','

typedef struct {
	char 	adminname[UserLen+1];		// user name 
	char	adminpw[PassLen+1];		// password
	int		adm_timeout;		// Mins
} ADMIN_CONF;

typedef struct{
	char name[UserLen+1];			// user name 
	char password[PassLen+1];		// password
	
} USER_CONF1;

typedef struct {
	int	enable;				// enable or disable 
	int	max_user;			// maximum number of user
	int	cur_user;			// current number of user
	USER_CONF1 user_list[USER_MAX_USER_NUM]; //
} USER_CONF;

typedef struct{
	USER_CONF	uconf;
	ADMIN_CONF	admconf;
	char	 	user_info[USER_MAX_USER_NUM][AllLen+2];	//add "/" 
} USER_LIST;



typedef USER_CONF	USER_INFO;

#ifdef _HAS_VIEWER_ACCOUNT_
typedef	struct{
	char	name[UserLen+1];
	char	pswd[PassLen+1];
} VIEWER_INFO;
#endif

typedef struct{
	ADMIN_CONF	adminfo;
#ifdef _HAS_VIEWER_ACCOUNT_	
	VIEWER_INFO	vinfo;
#endif
	USER_INFO	uinfo;
	int		audio_in_ctrl;
	int		audio_out_ctrl;
	int		io_ctrl;
	int		adm_ctrl;
} USER_CFG;

int check_user_data(char *user_item,char *name,char *pass);

//------------------------------------------------------------
//purpose: Read user info from file /etc/system.conf
//output: buffer with user info
//return: User_OK	- succeed	
//	  others - fail
//------------------------------------------------------------
int ReadUserConf(USER_INFO *pconf);

//------------------------------------------------------------
//purpose: Set user info to file /etc/system.conf
//input: enable - 1:enable user setting, 0:disable user setting
//return: User_OK	- succeed	
//	  others - fail
//------------------------------------------------------------
int SaveUserConf(USER_INFO *uinfo);

//------------------------------------------------------------
//purpose: Get user info
//input: start - start index (default 0)
//       num - how many users to get
//output: user - buffer to save user info, buffer size is equal
//               to sizeof(USER_CONF1)*num 
//return: User_OK	- succeed	
//	  others - fail
//------------------------------------------------------------
int ReadUsers(int start,int num,USER_CONF1 user[]);


//--------------------------------------------------------
//purpose: Read administrator info from /etc/system.conf 
//output: buffer with administrator info
//return: User_OK - succeed
//	  others - fail
//-------------------------------------------------------
int ReadAdminConf(ADMIN_CONF *pconf);

//--------------------------------------------------------
//purpose: Set administrator info from /etc/system.conf 
//input: buffer with administrator info
//return: User_OK - succeed
//	  others - fail
//note: naming rule : the same with AddUser()
//--------------------------------------------------------
int SaveAdminConf(ADMIN_CONF *pconf);

//--------------------------------------------------------
//purpose: Set administrator info from /etc/def_sys.conf 
//	   to /etc/system.conf
//return: User_OK - succeed
//	  others - fail
//--------------------------------------------------------
int SetDefaultAdmin(void);
#ifdef _SC_DEFAULT_USERNAMEPASS_BUTTON_3SEC_
int SetDefaultUser(void);
#endif


//--------------------------------------------------------
//purpose: Get all config in configuration file
//input: pointer of the structure contains all user config
//return: User_OK - succeed
//	  others - fail
//--------------------------------------------------------
int USER_RdAllCfg(USER_CFG* user_cfg);

//--------------------------------------------------------
//purpose: Set all config in configuration file
//input: pointer of the structure contains all user config
//return: User_OK - succeed
//	  others - fail
//--------------------------------------------------------
int USER_WrAllCfg(USER_CFG* user_cfg);

int ReadAllUserAdminPerm(int *perm);
int SetAllUserAdminPerm(int *perm);
int ReadAllUserIOPerm(int *perm);
int SetAllUserIOPerm(int *perm);
int ReadAllUserAudioInPerm(int *perm);
int SetAllUserAudioInPerm(int *perm);
int ReadAllUserAudioOutPerm(int *perm);
int SetAllUserAudioOutPerm(int *perm);
int ReadAllUserPTPerm(int *perm);
int SetAllUserPTPerm(int *perm);



//return error
#define	User_OK				0
#define	User_InputParamErr		-1
#define	User_ReadUserConfErr		-2
#define	User_ReadAdminConfErr		-3
#define User_SaveUserConfErr		-4
#define User_SaveAdminConfErr		-5

#define User_ReadUsersErr		-6
#define User_ModifyUserErr		-7
#define User_AddUserErr			-8
#define User_AddUserDupErr		-9
#define User_AddUserOverErr		-10
#define User_DelUserErr			-11
#define User_ReadAdminDefErr		-12
#define User_DelAllUsersErr			-13

int user_dup_check(USER_CFG	*user_conf);

#endif

