#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/un.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <ctype.h>

extern int errno;

#include "config.h"
#include "libhttpd.h"
#include "common.h"
#include "sc_tool.h"
#include "sc_debug.h"


int pass_fd_and_message(int fd, client_message *p_msg) {
	int ret;
	int s;			/* socket */
	struct iovec iov[1];
	struct msghdr msg;
	struct cmsghdr *cmsgp = NULL;
	char buf[CMSG_SPACE (sizeof fd)];
	struct sockaddr_un a_srvr;	/* serv. adr */

	/* Clear message area */

	memset (&a_srvr, 0, sizeof a_srvr);
	memset (&msg, 0, sizeof (msg));
	memset (buf, 0, sizeof (buf));

	/* Create a Unix Socket */
	s = socket (PF_LOCAL, SOCK_STREAM, 0);

	if (s == -1)
	{
		printf (" create socket error\n");
		return -1;	/* Failed: check errno */
	}

	/* Create the abstract address of the socket server */
	a_srvr.sun_family = AF_LOCAL;
	strncpy (a_srvr.sun_path, "./zSOCKET-SERVER",
		 sizeof a_srvr.sun_path - 1);
	a_srvr.sun_path[0] = 0;

	/* Connect to the sock server */
	ret = connect (s, (struct sockaddr *) &a_srvr, sizeof a_srvr);

	if (ret == -1)
	{
		printf ("connect failed\n");
		return -1;	/* Failed: check errno */
	}


	/* supply socket addr (if any) */

	msg.msg_name = NULL;
	msg.msg_namelen = 0;


	/* init I/O vector to send the value in er. 
	 * This is done because data must be transmitted to send 
	 * the fd 
	 */

	iov[0].iov_base = p_msg;
	iov[0].iov_len = sizeof(client_message);
	msg.msg_iov = &iov[0];
	msg.msg_iovlen = 1;

	/* Establish control buffer */
	msg.msg_control = buf;
	msg.msg_controllen = sizeof buf;

	/* configure the message to send a fd */
	cmsgp = CMSG_FIRSTHDR (&msg);
	cmsgp->cmsg_level = SOL_SOCKET;
	cmsgp->cmsg_type = SCM_RIGHTS;
	cmsgp->cmsg_len = CMSG_LEN (sizeof fd);

	/* install the file descriptor value */
	*((int *) CMSG_DATA (cmsgp)) = fd;
	//msg.msg_controllen = cmsgp->cmsg_len;

	/* send it to client process */
	do {
		ret = sendmsg (s, &msg, 0);
	}
	while (ret == -1 && errno == EINTR);

	// close socket before exit 
	close (s);
	return ret == -1 ? -1 : 0;
}
/*
function's name: parse_int_parameter
input:
	char *cmd - the string need to parse
	char *para - the parameter in cmd(format is : "key_name=")
	int value_max_len - para's value's max length
	int error_value - if have error,out_val will be set to error_value.
output:
	int *out_val - para's value output
return:
	1 - ok,and need get value from out_val
	0 - ok,don't get value from out_val(it is wrong)
   -1 - error, need set para's value to error_value
*/
static int parse_int_parameter(char *cmd,char *para,int default_val,int *out_val)
{
#define MAX_URL_KEY_VALUE_LEN	32
#define ERROR_VALUE				128
	char *p=cmd,*e;
	char re_str[MAX_URL_KEY_VALUE_LEN+1];
	int i=0,t=0;

	if (NULL == out_val)
	{
		return -1;
	}
	if (NULL == cmd || NULL == para)
	{
		*out_val = default_val;
		return 0;
	}
	while(1)
	{
		if((p=strstr(p,para)))
		{
			if(*(p-1)!='&' && p!=cmd)
			{
				if(NULL==(p=strstr(p,"&")))
				{
					*out_val = default_val;
					return 0;
				}
				continue;
			}
			else
			{
				break;
			}
		}
		else
		{
			*out_val = default_val;
			return 0;
		}
		
	};
	strncpy(re_str, p+strlen(para), MAX_URL_KEY_VALUE_LEN);
	re_str[MAX_URL_KEY_VALUE_LEN] = 0;
	if ((e=strstr(re_str, "&")))
	{
		*(e) = 0;
	}
	t=strlen(re_str);
	for(i=0;i<t;i++)
	{
		if (!isdigit(re_str[i]))
		{
			*out_val=ERROR_VALUE;
			return -1;
		}
	}
	*out_val = atoi(re_str);
	return 0;
}


void sc_handle_send_video(httpd_conn *hc)
{
	int ccode;
	char *p_user = NULL;
	char *p_addr = NULL;
	char *p_agent = NULL;
	
	client_message message;
	memset(&message, 0, sizeof(message));

	p_addr = httpd_ntoa(&hc->client_addr);
	if(p_addr != NULL) {
		strcpy(message.ip, p_addr);
	}

	p_user = hc->remoteuser;
	if(p_user != NULL) {
		strncpy(message.name, p_user, 32);
		message.name[31] = 0;
	}
	if(0 == message.name[0])
		strcpy(message.name, "anonymous");
	
	if (METHOD_GET == hc->method)
	{
		message.method = REQ_METHOD_GET;
	}
	else
	{
		message.method = REQ_METHOD_POST;
	}
	parse_int_parameter(hc->query,"size=",0,&message.resolution);
	parse_int_parameter(hc->query,"framerate=",0,&message.framerate);
	p_agent = hc->useragent;
	if(p_agent != NULL) {		
#if 1
		if(strstr(p_agent, "MSIE") != NULL) 
		{
			message.agent = USER_AGENT_IE;
		}
		else if(strncmp(p_agent, "CameraActiveX", 13) == 0 || 
				strncmp(p_agent, "Viewer", 6) == 0 || 
				strncmp(p_agent, "AlertCfg", 8) == 0) 
		{
			message.agent = USER_AGENT_OCX;
		}
		else 
		{
			message.agent = USER_AGENT_MOZILLA;
		}

#else
		if(strstr(p_agent, "MSIE") != NULL) 
		{
			message.agent = USER_AGENT_IE;
		}
		else if(strstr(p_agent, "Mozilla") != NULL)
		{
			message.agent = USER_AGENT_MOZILLA;
		}
		else 
		{
			message.agent = USER_AGENT_OCX;
		}
#endif
	
	}

	if(strstr(hc->origfilename, "jpeg") != NULL) 
	{
		message.video_type = VIDEO_TYPE_JPEG;
	}
	else 
	{
		message.video_type = VIDEO_TYPE_ASF;
	}

	ccode = pass_fd_and_message(hc->conn_fd, &message);
	//close(hc->conn_fd);	// Halcyon, don't close here, thttpd will clear this session

	fprintf(stderr, "(c=%d) %u %u\n", ccode, message.video_type, message.agent);
}

#if 0
void sc_handle_send_video(httpd_conn *hc)
{
	int ccode;
	char user_message[128];

	char c;
	char *p_user = NULL;
	char *p_addr = NULL;
#ifdef _PASS_USER_AGENT_TO_SOCKSERV_
	char d;
	char *p_agent = NULL;
#endif

	p_addr = httpd_ntoa(&hc->client_addr);
	p_user = hc->remoteuser;
#ifdef	_PASS_USER_AGENT_TO_SOCKSERV_
	p_agent = hc->useragent;
#endif

	printf("get sockserv pid\n");
	ccode = GetAppsPid("sockserv");
	if(ccode == 0) {
		close(hc->conn_fd);
		DBG("sockserv is close.\n");
		return;
	}


#ifdef _PASS_USER_AGENT_TO_SOCKSERV_
	if ((NULL != p_agent) && (0 == strncmp(p_agent, "Mozilla", 7))) {
		d = 'b';
	} else {
		d = 'a';
	}
#endif	

	if (0) {
		c = 'a';
	} else {
		c = 'b';
	}

	if((NULL != p_user) && (strlen(p_user) != 0)) {
#ifdef _PASS_USER_AGENT_TO_SOCKSERV_
		sprintf(user_message, "%c%c%s, %s", c, d, p_addr, p_user);
#else
		sprintf(user_message, "%c%s, %s", c, p_addr, p_user);
#endif
	} else {
#ifdef _PASS_USER_AGENT_TO_SOCKSERV_
		sprintf(user_message, "%c%c%s", c, d, p_addr);
#else
		sprintf(user_message, "%c%s", c, p_addr);
#endif
	}

	DBG("%s\n", user_message);

	ccode = pass_fd_and_message(hc->conn_fd, user_message);

	close(hc->conn_fd);
}


static int sk_pass_fd(int fd, char *msgbuf)
{
	int ret;
	int s;			/* socket */
	struct iovec iov[1];
	struct msghdr msg;
	struct cmsghdr *cmsgp = NULL;
	char buf[CMSG_SPACE (sizeof fd)];
	struct sockaddr_un a_srvr;	/* serv. adr */

	/* Clear message area */

	memset (&a_srvr, 0, sizeof a_srvr);
	memset (&msg, 0, sizeof (msg));
	memset (buf, 0, sizeof (buf));

	/* Create a Unix Socket */
	s = socket (PF_LOCAL, SOCK_STREAM, 0);

	if (s == -1)
	{
		DBG("create socket error\n");
		return -1;	/* Failed: check errno */
	}

	/* Create the abstract address of the socket server */
	a_srvr.sun_family = AF_LOCAL;
	strncpy (a_srvr.sun_path, "./zSOCKET-SERVER",
		 sizeof a_srvr.sun_path - 1);
	a_srvr.sun_path[0] = 0;

	/* Connect to the sock server */
	ret = connect (s, (struct sockaddr *) &a_srvr, sizeof a_srvr);

	//DBG("Connect to server s=%d ret=%d\n", s, ret);
	if (ret == -1)
	{
			DBG("connect failed\n");
			return -1;	/* Failed: check errno */
		}


		/* supply socket addr (if any) */

		msg.msg_name = NULL;
		msg.msg_namelen = 0;


		/* init I/O vector to send the value in er. 
		 * This is done because data must be transmitted to send 
		 * the fd 
		 */

		iov[0].iov_base = msgbuf;
		iov[0].iov_len = strlen (msgbuf);
		msg.msg_iov = &iov[0];
		msg.msg_iovlen = 1;

		/* Establish control buffer */
		msg.msg_control = buf;
		msg.msg_controllen = sizeof buf;

		/* configure the message to send a fd */
		cmsgp = CMSG_FIRSTHDR (&msg);
		cmsgp->cmsg_level = SOL_SOCKET;
		cmsgp->cmsg_type = SCM_RIGHTS;
		cmsgp->cmsg_len = CMSG_LEN (sizeof fd);

		/* install the file descriptor value */
		*((int *) CMSG_DATA (cmsgp)) = fd;
		//msg.msg_controllen = cmsgp->cmsg_len;
		DBG("msgbuf: %s \n", (char *)iov[0].iov_base);

		/* send it to client process */
		do
		{
			ret = sendmsg (s, &msg, 0);
		}
		while (ret == -1 && errno == EINTR);

		// close socket before exit 
		if(close (s))
			DBG("close failed, reason %s\n",strerror(errno));
		return ret == -1 ? -1 : 0;

	}
#endif

void sc_handle_send_rtsp(httpd_conn* hc)
{
	int ccode;
	client_message message;
	char buf[32];
	int postlen;

	memset(&message, 0, sizeof (client_message));
	sprintf( buf, "%s:%d", httpd_ntoa(&hc->client_addr), ntohs(hc->client_addr.sa_in.sin_port));
	strcpy( message.ip , buf);
	if(hc->remoteuser != NULL)
	{	
		strncpy(message.name, hc->remoteuser, 32);
		message.name[31] = 0;
	}
	if(0 == message.name[0])
		strcpy(message.name, "anonymous");
		
	SC_DBG_WEB("%s(%d)message.name=%s\n", __FUNCTION__, __LINE__, message.name);
	if (METHOD_GET == hc->method)
	{
		message.method = REQ_METHOD_GET;
	}
	else
	{
		message.method = REQ_METHOD_POST;
	}
	message.resolution = RESOLUTION_NONE;
	message.agent = USER_AGENT_NONE;
	message.video_type = VIDEO_TYPE_SAV;
	message.quality = QUALITY_NONE;
	strncpy(message.x_session_cookie, hc->x_sessioncookie, 32);
	message.x_session_cookie[31] = 0;

	/* copy post data */
	postlen = hc->read_idx - hc->checked_idx;
	postlen = postlen > sizeof(message.post_data) ? sizeof(message.post_data) : postlen;
	memcpy(message.post_data, &(hc->read_buf[hc->checked_idx]), postlen);
	message.post_data_len = postlen;

	/* now issue our request */
	ccode = pass_fd_and_message(hc->conn_fd, &message);
	fprintf(stderr, "pass fd (c=%d)\n", ccode);

// Halcyon, don't close here, thttpd will clear this session
/*
	if (close(hc->conn_fd)) 
			DBG("close failed, reason %s\n",strerror(errno));
*/	
	return;
}


